function Iterator(resource) {
	this.res = resource;
	this.index = 0;
	this.curr = 0;

	this.hasNext = function() {
		return this.index < this.res.length;
	}
	this.next = function() {
		this.curr = this.res[this.index++];
		return this.curr;
	}
	this.peek = function() {
		return this.curr;
	}
}

function ConstIterator(value) {
	this.curr = value;

	this.hasNext = function() {
		return true;
	}
	this.next = function() {
		return this.curr;
	}
	this.peek = function() {
		return this.curr;
	}
}

function IntsIterator(from, to) {
	this.curr = from;
	this.end = to;

	this.hasNext = function() {
		return this.curr <= this.end;
	}
	this.next = function() {
		return this.curr++;
	}
	this.peek = function() {
		return this.curr;
	}
}

function AddIterator(i1, i2) {
	this.i1 = i1;
	this.i2 = i2;

	this.hasNext = function() {
		return i1.hasNext() || i2.hasNext();
	}
	this.next = function() {
		var v1 = i1.hasNext() ? i1.next() : 0;
		var v2 = i2.hasNext() ? i2.next() : 0;
		return this.curr = v1 + v2;
	}
	this.peek = function() {
		return this.curr;
	}
}

function IdentityIterator(iterator, functor, data) {
	this.inner = iterator;

	this.hasNext = function() {
		return this.inner.hasNext();
	}
	this.next = function() {
		return this.inner.peek(); // YES - peek()
	}
	this.peek = function() {
		return this.inner.peek();
	}
}

function MapIterator(iterator, functor, data) {
	this.inner = iterator;
	this.curr = 0;

	this.hasNext = function() {
		return this.inner.hasNext();
	}
	this.next = function() {
		this.curr = functor(this.inner.next(), data);
		return this.curr;
	}
	this.peek = function() {
		return this.curr;
	}
}

function Series(resource) {
	this.res = resource;

	this.getIterator = function() {
		return new Iterator(this.res);
	}
}

function ConstSeries(resource) {
	this.res = resource;

	this.getIterator = function() {
		return new ConstIterator(this.res);
	}
}

function IntsSeries(from, to) {
	this.from = from;
	this.to = to;

	this.getIterator = function() {
		return new IntsIterator(this.from, this.to);
	}
}

function AddSeries(s1, s2) {
	this.s1 = s1;
	this.s2 = s2;

	this.getIterator = function() {
		return new AddIterator(this.s1.getIterator(), this.s2.getIterator());
	}
}

function TransposedIterator(iteratorArray) {
	this.its = iteratorArray;

	this.hasNext = function() {
		for(i in this.its) {
			if (this.its[i].hasNext()) return true;
		}
		return false;
	}
	this.next = function() {
		var d = new Array();
		for(i in this.its) {
			if (this.its[i].hasNext())
				d.push(this.its[i].next());	
			else
				d.push(null);
		}
		alert(dump(d));
		return this.curr = new Series(d);
	}
	this.peek = function() {
		return this.curr;
	}
}

function TransposedSeries(seriesArray) {
	// get iterators
	var its = new Array();
	for(s in seriesArray) {
		its.push(seriesArray[s].getIterator());
	}

	// construct tranposed series array 't'
	this.res = new Array();
	do {
		var d = new Array();
		hasData = false;
		for(i in its) {
			if (its[i].hasNext()) {
				hasData = true;
				d.push(its[i].next());	
			} else
				d.push(null);
		}
		if (!hasData) break;
		this.res.push(new Series(d));
	} while(true);

	this.getIterator = function() {
		return new Iterator(this.res);
	}
}



/**
 * Dependency series...
 **/

function IteratingSeries(series) {
	this.base = series;
	this.idx = -1;

	this.getIterator = function() {
		this.idx = -1;

		return new MapIterator(this.base.getIterator(), 
		                       function(v, series) { series.idx++; return v; },
		                       this);
	}
	this.getIndex = function() {
		return this.idx;
	}
}

function ControlledSeries(series, baseSeries) {
	this.ser = series;
	this.base = baseSeries;

	this.getIterator = function() {
		// FIXME: HACK accessing the series resource directly (needs redesign)
		if (this.ser.res[this.base.getIndex()] == undefined) {
			alert("ConstrolledSeries: undef iterator: @" + this.base.getIndex() + "\nser.res: " + this.ser.res + "\nbase.res: " + this.base.res);
			return null;
		}
		return this.ser.res[this.base.getIndex()].getIterator();
	}
}

function InspectedSeries(series, over) {

	function inspectSeriesBy(series, over) {
		if (!over || !over.length) {
			return new SeriesProperties().inspect(series.getIterator());
		}

		var props = new SeriesProperties();
		var o = over.shift(); //FIXME: IE?
		var iter = o.getIterator();
		while(iter.hasNext()) {
			iter.next();
			props.merge( inspectSeriesBy(series, over) );
		}
		return props;
	}

	function inspectSeries(series, over) {
		// use a copy of the 'over' array
		var o = new Array();
		for (var property in over) o[property] = over[property];
		return inspectSeriesBy(series, o);
	}

	this.getIterator = function() {
		return series.getIterator();
	}

	this.props = inspectSeries(this, over);
}

/**
 * Flattened series iterator contains list data items collected in
 * a DFS series hierarchy tree walk.
 **/

function FlattenIterator(series) {
	this.res = new Array();
	this.index = 0;
	this.curr = 0;

	function pushItem(container, item) {
		if (item == null || item.getIterator == undefined) {
			container.push(item);
			return;
		}
		// recursive iterator flatten
		var it = item.getIterator();
		while(it.hasNext()) {
			pushItem(container, it.next());
		}
	}
	pushItem(this.res, series);

	this.hasNext = function() {
		return this.index < this.res.length;
	}
	this.next = function() {
		this.curr = this.res[this.index++];
		return this.curr;
	}
	this.peek = function() {
		return this.curr;
	}
}

function FlattenedSeries(series) {
	this.ser = series;

	this.getIterator = function() {
		return new FlattenIterator(this.ser);
	}
}


/**
 * Metric inspection.
 **/

/**
 * Series inspection data representation object.
 **/
function SeriesProperties() {
	this.count = 0;

	/**
	 * Conveniently merge two series properties.
	 * @param other SeriesProperties instance to merge info from
	 **/
	function merge(other) {
		if (!this.count) {
			this.count = other.count;
			this.sum = other.sum;
			this.min = other.min;
			this.max = other.max;
			return this;
		}

		this.count = this.count > other.count ? this.count : other.count;
		this.sum = this.sum > other.sum ? this.sum : other.sum;
		this.min = this.min < other.min ? this.min : other.min;
		this.max = this.max > other.max ? this.max : other.max;
		return this;
	}

	function mergeElement(e) {
		if (!this.count) {
			this.count = 1;
			this.sum = e;
			this.min = e;
			this.max = e;
			return e;
		}

		this.count++;
		this.sum += e;
		this.min = this.min < e ? this.min : e;
		this.max = this.max > e ? this.max : e;
		return e;
	}

	function inspect(it) {
		if (!it || !it.hasNext()) {
			return this;
		}

		var count = 1;
		var sum = min;
		var min = it.next();
		var max = min;
		while(it.hasNext()) {
			var e = it.next();
			count++;
			sum += e;
			min = min<e ? min:e;
			max = max>e ? max:e;
		}

		this.count = count;
		this.sum = sum;
		this.min = min;
		this.max = max;
		return this;
	}

	// public members
	this.merge = merge;
	this.mergeElement = mergeElement;
	this.inspect = inspect;
}


function inspect(it) {
	if (! it.hasNext()) {
		return new SeriesProperties();
	}
		
	var min = it.next();
	var max = min;
	var sum = min;
	var count = 1;
	while(it.hasNext()) {
		var e = it.next();
		sum += e;
		min = min<e ? min:e;
		max = max>e ? max:e;
		count++;
	}

	return { min: min, max: max, sum: sum, count: count };
}


// FIXME: so far unused...

function NeighborSumIterator(iterator) {
	return new MapIterator(iterator,
			function(v, data) {
		var r = v + data.prev;
		data.prev = v;
		return r;
	}, {
		prev :iterator.next()
	});
}

// vim: ai:ts=4:sw=4:tw=0
