/**
TODO:

- make it 2 pass process:
   a) calculations b) rendering using the calculated data

* all metrics should get inspect()ed, so that the max, min etc are known
  and are calculated across all the necessary graph tree branches
* uniform access to attributes and metrics in the code
  - transform non-metric series into metrics (IntsSeries or something)
    so that they could be used directly

***/

dojo.require("dojox.gfx.utils"); 

var charting = {
    setCarry : function(cc, pairs) {
		for(p in pairs) {
			cc[p] = pairs[p].getIterator();
		}
    },
    carry : function(name) {
		return function() {
			// @note this.class = Ctx
			var it = this.get(name)[name];
			if (it.hasNext())
				return it.next();
			return null;
		}
    },

    ecLabel : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);

		// get by-ref value
		var val = chart.value != undefined ? chart.value.call(ctx) : '[none]';
		if (val == null)
			val = '[NULL]';

	    var e = new gfx.Text(ctx, val).render().setStroke(s);
		//e.children[0].setFont({size:'16px'});
		if (f != undefined) e.setFill(f);
		// place it to the center of the designated space
	    return tr.move(e, (c.w-e.getTextWidth()-2) / 2, c.h/2 - 5);
    },

    ecBar : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);
		var e = new gfx.Bar(ctx, c.w, c.h).render().setStroke(s);
		if (f != undefined) e.setFill(f);
		return e;
    },

    ecLine : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
	    var ph = ctx.prev.height;
	    ctx.prev.height = c.height.peek();
	    return new gfx.Line(ctx, c.width.peek(), c.height.peek() - ph).render();
//	    return new gfx.Line(ctx, -c.width.peek() / 2, ph - c.height.peek(),
//                c.width.peek(), c.height.peek() - ph, s
//                        .peek()).render();
    },

    ecEllipse : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);
		var e = new gfx.Ellipse(ctx, c.w/2, c.h/2).render().setStroke(s);
		// place it to the center of the designated space
		var e = tr.move(e, c.w/2, c.h/2);
		if (f != undefined) e.setFill(f);
		return e;
    },

    ecPieSlice : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);
		var e = new gfx.PieSlice(ctx, c.angle.peek(), c.radius.peek()).render()
					.setStroke(s);
		if (f != undefined) e.setFill(f);
		return e;
    },

    ecPie: function(ctx,chart) {
    	var d = chart.series['width'];
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

    	// calculate the sum of all segments
    	var props = inspect( d.getIterator());
    	if (props.sum <= 0) {return '';}

    	c.colors = chart.series['colors'].getIterator();

    	// render separate slices
    	c.width = d.getIterator();
    	c.angle = new MapIterator( d.getIterator(), function(v, sum) { return Math.round((v*360)/sum); }, props.sum );
    	// c.radius = new ConstIterator(c.parent.w/2);
    	//c.radius = new MapIterator(d.getIterator(), function(v, p) { return v*p.w/p.sum; }, {w: c.parent.w, sum: props.sum} );

    	var dummy = function(i) {
    		//alert(i);
    	};

		// FIXME: HACK get the radius from a provided dimension rather then making it up
    	c.radius = new MapIterator(d.getIterator(), function(v, p) {
    								dummy(p.size + "/" + p.sum + " = " + p.w*(p.size/p.sum)); return p.w/2*v; },
    								{size: 2, w: c.parent.w, sum: props.sum} );
    								//{size: c.parent.size.peek(), w: c.parent.w, sum: props.sum} );
    	//c.radius = new MapIterator(d.getIterator(), function(v, p) { return p.w*(p.size/7); }, {size: c.parent.size.peek(), w: c.parent.w, sum: props.sum} );

    	var ret = c.group;
    	while(c.angle.hasNext()) {
    		c.width.next();
    		c.radius.next();
    		c.style = {fill: c.colors.next()};
    		var a = c.angle.next();

    		// rotate the parent group of slices
    		tr.rotate(c.group,a);

    		//Switch the g and c.group in the DOM
    		// parent -> g -> c.group
    		// parent -> c.group -> g
    		g = c.group;
    		var parent = g.getParent();
    		c.group = new gfx.Group(ctx).render();
    		c.group.removeShape();
    		parent.add(c.group);
    		c.group.add(g);
    		ret = c.group;
    		
    		// draw the current slice into its own subgroup
    		this.ec(ctx, chart.what);
    	}
    	ctx.pop();
    	return ret;
    },

    ecShape : function(ctx, chart) {
    	switch(chart.which) {
    	case 'line': 
    		return this.ecLine(ctx, chart);
    	case 'bar': 
    		return this.ecBar(ctx, chart);
    	case 'ellipse': 
    		return this.ecEllipse(ctx, chart);
		}
    },

    ecColumns: function(ctx, chart) {
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

    	// calculate the max of all segments
    	var d = chart.series['height'];
    	values = d!=null ? d.getIterator() : null;

		// No series for the column 'height' -> empty group
		if (values == null) {
    		ctx.pop();
			return g;
		}
    	var props = inspect(values);
    	var dy = c.parent.h/props.max;

    	ctx.prev = {};
    	c.colors = chart.series['colors'].getIterator();

		this.setCarry(c, chart.carry);

    	values = d.getIterator();

    	// HMMM....
    	c.size = chart.series['size'];
    	c.size = c.size != undefined
					? c.size.getIterator() : new IdentityIterator(values);

    	// AVOID: using multiple iterators over the same series
    	//        it could confuse the ControlledSeries
    	//
    	//c.height = new MapIterator(
		//					d.getIterator(),
    	//					function(v, p) { return v*p.dy+p.offset; },
    	//					{ dy:dy, offset:500 });
    	var widthSeries = new ConstSeries(c.parent.w/props.count - chart.pad);
    	c.width = widthSeries.getIterator();
    	c.offset = new MapIterator(widthSeries.getIterator(),
    	                           function(v, pad) { return v+pad; },
    	                           chart.pad );
    	// c.width = new MapIterator(
		//				new ConstSeries(60).getIterator(),
		//				function(v, seq) { seq.value++; return v*seq.value; },
		//				{ value:0 } );

    	// iterate over the chart values
    	var ret = c.group;
    	var n = 0;
    	while(values.hasNext()) {
    		var h = values.next() * dy;
    		c.width.next();
    		c.size.next();
    		c.style = {fill: c.colors.next()};

    		c.w = c.width.peek();
    		c.h = h;

				tr.move(this.ec(ctx, chart.what),
						c.offset.next() * n, 0);
			//FIXME: excluding some values throws off the 'carry' iterators
			//if (values.peek() != null) {
			//}
			n++;
		}

		ctx.pop();
    	return ret;
    },

    ecRows: function(ctx, chart) {
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

    	// calculate the max of all segments
    	var d = chart.series['size'];
    	sizes = d!=null ? d.getIterator() : null;

		// No series for the column 'height' -> empty group
		if (sizes == null) {
    		ctx.pop();
			return g;
		}
    	var props = inspect(sizes);
    	var dy = c.parent.h/props.sum;

    	c.colors = chart.series['colors'].getIterator();

		this.setCarry(c, chart.carry);

    	sizes = d.getIterator();

    	var offset = 0;
    	while(sizes.hasNext()) {
    		var h = sizes.next() * dy;

    		c.style = {fill: c.colors.next()};
    		c.w = c.parent.w;
    		c.h = h;

				tr.move(this.ec(ctx, chart.what), 0, offset);
			//FIXME: excluding some values throws off the 'carry' iterators
			//if (values.peek() != null) {
			//}
			offset += h;
		}

		ctx.pop();
    	return g;
    },

    ecScattered: function(ctx, chart, lines) {
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

		// FIXME: These should be calculated from the size of the
		//        markers + some defaults to get 'padding'
		var xmin = c.parent.w/15;
		var ymin = c.parent.w/15;

    	// calculate the max of all segments
    	var x = chart.series['x'];
    	var y = chart.series['y'];

    	var propsX = inspect(x.getIterator());
    	var dx = (c.parent.w-xmin*2)/propsX.max;

    	var propsY = inspect(y.getIterator());
    	var dy = (c.parent.h-ymin*2)/propsY.max;

    	c.x = x.getIterator();
    	c.y = y.getIterator();
		c.style = chart.style;

    	// iterate over the chart values
		var ret = c.group;
		var path = (lines)
					? new gfx.Path(ctx).render().setStroke(c.style) : null;
		var n = 0;
		while(c.x.hasNext() && c.y.hasNext()) {
			var px = xmin + c.x.next() * dx;
			var py = ymin + c.y.next() * dy;

			if (c.x.peek() != null && c.y.peek() != null) {
				if (lines) {
					if (n > 0)
						path.lineTo(px,py);
					else
						path.moveTo(px,py);
				}

				c.w = Math.min(c.parent.w - px, px);
				c.h = Math.min(c.parent.h - py, py);
				// FIXME: Hack: how big should the markers be?
				c.w = c.parent.w/12;
				c.h = c.parent.w/12;

				tr.move(this.ec(ctx, chart.what), px-c.w/2, py-c.h/2);
			}
			n++;
		}
		ctx.pop();
    	return ret;
    },


    ecGrid: function(ctx, chart) {
    	var cols = chart.series['cols'];
    	var rows = chart.series['rows'];
    	var cellstyles = chart.series['cell_styles'];
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;
    	var propsC = inspect( cols.getIterator());
    	var propsR = inspect( rows.getIterator());
    	var widthSeries = new ConstSeries(c.parent.w/propsC.count - chart.pad*2);
    	var heightSeries = new ConstSeries(c.parent.h/propsR.count - chart.pad*2);

    	var gridlines = chart.gridlines != undefined
							? chart.gridlines : false;

		this.setCarry(c, chart.carry);

		var ret = c.group;

		/**
		// Grid's bounding rectangle...

		new gfx.Path(ctx, "black").render().setStroke(gridlines)
				.moveTo(0,0).hLineTo(c.parent.w).vLineTo(c.parent.h)
				.hLineTo(0).closePath();
		**/

    	var cellstylegroup = new gfx.Group(ctx).render();

		c.cellstyles = cellstyles != undefined
							? cellstyles.getIterator() : null;
    	c.heightOffset = new MapIterator(heightSeries.getIterator(),
    	                           function(v, pad) { return v+pad; },
    	                           chart.pad*2 );
		c.row = rows.getIterator();
		c.height = heightSeries.getIterator();
		var row = 1;
		while(c.row.hasNext()) {
			c.row.next();
			var yoff = c.heightOffset.next();
			c.h = c.height.next(); // propagate inner element height
			c.widthOffset = new MapIterator(widthSeries.getIterator(),
											function(v, pad) { return v+pad; },
											chart.pad*2 );
			c.width = widthSeries.getIterator();

			// horizontal grid lines
			if (gridlines && c.row.hasNext()) {
				var gl = new gfx.Line(ctx, c.parent.w, 0, 0).render();
				tr.move(gl, 0, c.parent.h - yoff * row);
				gl.setStroke(gridlines);
			}

			c.col = cols.getIterator();
			var col = 0;
			while(c.col.hasNext()) {
				c.col.next();
				var xoff = c.widthOffset.next();

				if (c.cellstyles) {
					var ggg = c.group;
					c.group = cellstylegroup;
					c.cellstyle = {width:0, fill: c.cellstyles.next()};

					// FIXME: HACK need to fix the style setup and other stuff
					var cg = new gfx.Bar(ctx, xoff, yoff).render();
					tr.move(cg,	xoff * col, -yoff * row + c.parent.h);
					cg.setStroke(c.cellstyle).setFill(c.cellstyle.fill),
					c.group = ggg;
				}

				// vertical grid lines
				if (gridlines && row == 1 && col > 0) {
					var gl = new gfx.Line(ctx, 0, c.parent.h, 0).render();
					tr.move(gl, xoff * col, 0);
					gl.setStroke(gridlines);
				}

				c.w = c.width.next(); // propagate inner element width

				// carry for dependent data series
				this.setCarry(c, chart.depcarry);

				tr.move(this.ec(ctx, chart.what),
						 xoff * col + chart.pad,
						-yoff * row + chart.pad + c.parent.h);
				col++;
			}
			row++;
		}

    	ctx.pop();
    	return ret;
    },


    ecOverlay : function(ctx, chart) {
    	var d = chart.series.getIterator();
    	var oldg = ctx.curr.group;
    	ctx.curr.group = new gfx.Group(ctx).render();
    	ret = ctx.curr.group;
    	while (d.hasNext()) {
    		this.ec(ctx, d.next());
    	}
    	ctx.curr.group = oldg;
    	return ret;
    },

    ec: function(ctx, chart) {
		if (!chart) return null;

    	switch(chart.type) {
    	case 'shape': 
    		return this.ecShape(ctx, chart);
    	case 'LINE': 
    		return this.ecLine(ctx, chart);
    	case 'BAR': 
    		return this.ecBar(ctx, chart);
    	case 'ELLIPSE': 
    		return this.ecEllipse(ctx, chart);
    	case 'PIESLICE': 
    		return this.ecPieSlice(ctx, chart);
    	case 'LABEL': 
    		return this.ecLabel(ctx, chart);
    	case 'pie': 
    		return this.ecPie(ctx, chart);
    	case 'grid': 
    		return this.ecGrid(ctx, chart);
    	case 'scattered': 
    		return this.ecScattered(ctx, chart, false);
    	case 'line': 
    		return this.ecScattered(ctx, chart, true);
    	case 'columns': 
    		return this.ecColumns(ctx, chart);
    	case 'rows': 
    		return this.ecRows(ctx, chart);
    	case 'overlay': 
    		return this.ecOverlay(ctx, chart);
    	}
    	return null;
    },

    ecc3: function(ctx, chart, w, h) {
		 return null;
    },

    ecc: function(ctx, chart, w, h) {
    	var c = ctx.push();

    	c.group = c.parent.group;
    	c.w = w;
    	c.h = h;

    	var ret = this.ec(ctx, chart);

    	ctx.pop();
    	return ret;
    }
}


var w = 800;
var h = 600;

var regions = new Series(['Central','East','South','West']);
var prod_types = new Series(['Coffe','Espresso','Herbal Tea','Tea']);

var coffe_prods = new Series(['Amaretto','Columbia','Decaf']);
var espresso_prods = new Series(['Cafe Latte','Mocca','Decaf Espresso', 'Espresso']);
var herbaltea_prods = new Series(['Cammomille','Lemon','Mint']);
var tea_prods = new Series(['Darjeeling','Earl Grey','Green Tea']);

var products = new Series([coffe_prods, espresso_prods, herbaltea_prods, tea_prods]);

var prod_types_it = new IteratingSeries(prod_types);
var regions_it = new IteratingSeries(regions);

var sales_c_c = new Series([14000,28900,26000]);
var sales_c_e = new Series([2994,47385,6262]);
var sales_c_s = new Series([null,21663,11596]);
var sales_c_w = new Series([9230,30300,18233]);

var sales_e_c = new Series([null,35000,24000,null]);
var sales_e_e = new Series([null,16646,7720,24000]);
var sales_e_s = new Series([15443,14166,15381,null]);
var sales_e_w = new Series([20466,18874,30578,null]);

var sales_coffe    = new Series([sales_c_c, sales_c_e, sales_c_s, sales_c_w]);
var sales_espresso = new Series([sales_e_c, sales_e_e, sales_e_s, sales_e_w]);

var sales = new Series([new ControlledSeries(sales_coffe, regions_it),
						new ControlledSeries(sales_espresso, regions_it)]);

var sales_ce = new Series([sales_coffe, sales_espresso]);

// Grid of line charts

var linegrid = {
    type :"grid",
    pad: 10,
    gridlines: {width:0.3, color:"darkblue"},
    series : {
		cols: new Series([1,2,3]),
		rows: new Series([1,2]),
    },
	what: {
   		type :"line",
		style: {width:5, color:'lightcoral', fill:'blue'},
    	series : {
			x: new Series([0,2.5,3,4,5]),
			y: new Series([2,5,3,4,0])
    	},
    	what : {
       		type :"shape",
       		which :"ellipse",
			style: {width:4, color:'gray'}
    	}
	}
}

var linegridgrid = {
    type :"grid",
    pad: 10,
    gridlines: {width:2, color:"green"},
    series : {
		cols: new Series([1,2]),
		rows: new Series([1,2,3]),
    },
	what: linegrid
}

var ctx = new Ctx();
	ctx.push().group = new gfx.Surface('svgdivc1', w, h).render();
var c1 = charting.ecc(ctx,linegridgrid,w,h);

// Grid of bar charts

prod_types = new Series(['Coffe','Espresso']);//,'Herbal Tea','Tea']);
prod_types_it = new IteratingSeries(prod_types);

var cellstyles = new Series([
		'white', 'lightyellow', 'yellow', 'orange',
		'darkorange', 'orangered', 'brown', 'purple', 'black']);

var cellstyle1 = new Series([
		'white', 'lightyellow', 'yellow', 'orange', 'darkorange']);
var cellstyle2 = new Series([
		'lightskyblue','lightcoral','lightgreen','lightyellow']);
var cellstyles_prod = new Series([cellstyle1, cellstyle2]); 

var cols_prodtype_region = {
	type :"columns",
	pad :10,
	series : {
		height: new ControlledSeries(sales, prod_types_it),
		// FIXME: TODO: height: charting.carry('val'),
		colors: new Series(['lightskyblue','lightcoral','lightgreen',
							'lightyellow','lightcyan','lightblack'])
		// legend: new ControlledSeries(products, prod_types_it)
	},
	what : {
		type :"overlay",
		series: new Series([{	type :"BAR",
								style:{width:3, color:'gray'}
							},
							{	type :"LABEL",
								style:{width:0, fill:'darkblue'},
								value: charting.carry('val')
							}])
	}
}

var colgrid = {
    type :"grid",
    pad :5,
    gridlines: {width:1, color:"blue"},
    series : {
        cols: prod_types_it,
        rows: regions_it,
    },
	depcarry: {
		val: new ControlledSeries(sales, prod_types_it)
	},
    what : cols_prodtype_region
}

var colgridgrid = {
    type: "grid",
    pad: 10,
    gridlines: {width:2, color:"green"},
    series : {
		cols: new Series([1,2]),
		rows: new Series([1]),
		cell_styles: cellstyles
    },
	what: colgrid
}

var ctx = new Ctx();
	ctx.push().group = new gfx.Surface('svgdivc2', w, h).render();
var c2 = charting.ecc(ctx,colgridgrid,w,h);


var slicepos = new IteratingSeries(new Series([1,0.5,2,1.4]));

var slices0 = new Series([3,1,1,2]);
var slices1 = new Series([1,2]);
var slices2 = new Series([1,1,2]);
var slices3 = new Series([3,1,2]);
var slicess = new Series([slices0,slices1,slices2,slices3]);
var slices = new ControlledSeries(slicess, slicepos);

var piecolors0 = new Series(["red","darkblue","green","blue"]);
var piecolors1 = new Series(["brown","pink","ivory","seablue"]);
var piecolors2 = new Series(["yellow","black","white","gray"]);
var piecolors3 = new Series(["red","darkblue","green","blue"]);
var piecolorss = new Series([piecolors0,piecolors1,piecolors2,piecolors3]);
var piecolors = new ControlledSeries(piecolorss, slicepos);
//var piecolors = new Series(["red","darkblue","green","blue"]);

// var slicepos = new IteratingSeries(new Series([2,1,2,2]));
slices.base = slicepos;
piecolors.base = slicepos;

var scatteredpies = {
   	type :"scattered",
	style: 'lightcoral',
   	series : {
		x: new Series([1,2,3,4,5]),
		y: slicepos
   	},
    what : {
        type :"pie",
        series : {
            width :slices,
            colors :piecolors
        },
        what : {
	        type :"PIESLICE",
			style: {width:2, color:"gray"},
        }
    }
}

var scatteredpiesgrid = {
    type :"grid",
    pad: 10,
    gridlines: {width:2, color:"green"},
    series : {
		cols: new Series([1,2]),
		rows: new Series([1,2,2]),
    },
	what: scatteredpies
}

var ctx = new Ctx();
	ctx.push().group = new gfx.Surface('svgdivc3', w, h).render();
var c3 = charting.ecc(ctx,scatteredpiesgrid,w,h);


var table = {
    type :"grid",
    pad :5,
    gridlines: {width:0, color:"black"},
    series : {
        cols: prod_types_it,
        rows: regions_it,
		cell_styles: cellstyles,
    },
    carry: {
		tabledata: new Series([10,20,30,40,50,60,70,80])
	},
    what : {
		type :"LABEL",
		value: charting.carry('tabledata'),
    	style: {color:"black", fill:"black"}
    }
}

var ctx = new Ctx();
	ctx.push().group = new gfx.Surface('svgdivc4', w, h).render();
var c4 = charting.ecc(ctx,table,w,h);


var v1 = new Series([10,20,150,40,50,60,70,80]);
var v2 = new Series([40,50,30,20,30,60,90,80]);
var v3 = new Series([30,70,10,20,30,60,70,80]);
var sums = new AddSeries(new AddSeries(v1,v2), v3);
var vt = new TransposedSeries([v1, v2, v3]);

//var sums_it = new IteratingSeries(new Series([1,1,1,1,1,1,1,1]));
var sums_it = new IteratingSeries(sums);
var v123t = new ControlledSeries(vt, sums_it);

var stackedbars = {
    type :"columns",
    pad :10,
    series : {
        height: sums_it,
        colors: new Series(["#006600"]) // not used - transparent columns
    },
	what: {
		type :"rows",
		series : {
			size: v123t,
			colors: new Series(['lightskyblue','lightcoral','lightgreen'])
		},
		carry: {
			val: v123t
		},
		what : {
			type :"overlay",
			series: new Series([{	type :"BAR",
									style:{width:3, color:'gray'}
								},
								{	type :"LABEL",
									style:{width:0, fill:'darkblue'},
									value: charting.carry('val')
								}])
		}
    }
}

var stackedbarsgrid = {
    type :"grid",
    pad: 10,
    gridlines: {width:2, color:"lightblue"},
    series : {
		cols: new Series([1,2]),
		rows: new Series([1,2]),
    },
	what: stackedbars
}

var ctx = new Ctx();
	ctx.push().group = new gfx.Surface('svgdivc5', w, h).render();
var c5 = charting.ecc(ctx,stackedbarsgrid,w,h);

var rows = {
	type :"rows",
	series : {
		size: new Series([1,2,3,4]),
		colors: new Series(['lightskyblue','lightcoral','lightgreen'])
	},
	carry: {
		val: new Series([1,2,3,4]),
	},
	what : {
		type :"overlay",
		series: new Series([{	type :"BAR",
								style:{width:3, color:'gray'}
							},
							{	type :"LABEL",
								style:{width:0, fill:'darkblue'},
								value: charting.carry('val')
							}])
	}
}

var ctx = new Ctx();
	ctx.push().group = new gfx.Surface('svgdivc6', w, h).render();
var c5 = charting.ecc(ctx,rows,w,h);


var piecharts = {
    type :"columns",
    pad :10,
    series : {
        height :slicepos,
        colors :piecolors  // unused for pie (should not need to enter here)
    },
    what : {
        type :"pie",
        series : {
            width :slices,
            colors :piecolors
        },
        what : {
	        type :"PIESLICE"
        }
    }
}

//slicepos.addListener(piecharts, 'height');
//slices.addListener(piecharts.what, 'width');

//var colpos = new IteratingSeries(new Series([1,1,1,1]));
var colpos = new IteratingSeries(new Series([1,0,6,3]));
var bars = new ControlledSeries(slicess, colpos);
var barcolors = new ControlledSeries(piecolorss, colpos);

var barcharts = {
    type :"columns",
    pad :10,
    series : {
        height :colpos,
        colors :barcolors
    },
    what : {
        type :"columns",
        pad :0,
        series : {
            height :bars,
            colors :barcolors
        },
        what : {
	        type :"BAR"
        }
    }
}

var barchart = {
    type :"columns",
    pad :10,
    series : {
        height :colpos,
        colors :new Series(["#006600","#008800","#00aa00","#00cc00"])
    },
    what : {
	    type :"BAR"
    }
}

var linechart = {
    type :"columns",
    pad :10,
    series : {
        height :slicepos,
        colors :piecolors
    },
    what : {
	    type :"LINE"
    }
}

var chart = {
    type :"overlay",
	series :new Series([ barchart, barcharts/*, linechart*/, piecharts])
}



var angle = 30;

function animate() {
	//angle = (angle + 10) % 360;

	tr.rotate(c1, angle);

	//alert(document.getElementById('svgDiv').innerHTML);
}

var animationId = 0;
function toggleAnimation() {
	if (animationId != 0) {
		clearInterval(animationId);
		animationId = 0;
	} else {
		animate();
		animationId = setInterval("animate()", 100);
	}
}

// vim: ai:ts=4:sw=4:tw=0
