/**
  How does it work:
  . set handlers
  . call calculate with the primary goal to inspect the carry metrics so that
	it is available for lower level charts 
	 - set current contexts?
	 - possibly transform attribute series into numbers
  . render using the inspect results


TODO by Jan 8th:
  . try to have the intermediate format
  . possibly do animations
  . presentation:
     - what are we aiming for

	 - HIGH LEVEL DESIGN
	    * it is recursive
		  - there is nothing out there in open-source world what would
		    do it for us - hey, this could be an advantage eventually
		* best bet is to have a declarative description of the shapes
		  - visualization tree declaration
		* data series entered are just objects which return a data
		  vector of numbers to render (no hierarchies, these are hidden in
		  the object implementations if necessary)

	 - LOW LEVEL DESIGN
	    * tree examples and their renderings
		* low level format overall purpose
		  - independent of the rendering engine used
		  - animations done here
		    . diff shape ID matching -> interpolation
		* browser options: SVG, VML, Flash, Silverlight
		  - doesn't matter really
		  - intended to be fairly simple to implement on top of anything
**/


var chartingNG = new function () {

	function carry(name) {
		return function() {
			// @note this.class = Ctx
			var it = this.get(name)[name+'_it'];
			if (it.hasNext())
				return it.next();
			return null;
		}
	}

	function CarryCtx() {
		function inspectSeriesBy(series, over) {
			if (!over || !over.length) {
				return new SeriesProperties().inspect(series.getIterator());
			}

			var o = over.shift();
			var iter = o.getIterator();
			var props = new SeriesProperties();
			while(iter.hasNext()) {
				iter.next();
				props.merge( inspectSeriesBy(series, over) );
			}
			return props;
		}

		function inspectSeries(series, over) {
			// create a new series properties member
			var o = new Array();
			for (var property in over)
				o[property] = over[property];
			return series.props = inspectSeriesBy(series, o);
		}

		function carryIterator(ctx, name) {
			var sctx = ctx.get(name);
			return sctx[name+'_it'] = sctx[name].getIterator();
		}

		function getIterators(ctx, pairs) {
			for(p in pairs) {
				carryIterator(ctx, p);
			}
		}

		function inspect(cc, pairs) {
			for(p in pairs) {
				// make sure the series contains its .props
				inspectSeries( pairs[p].series, pairs[p].over);

				// put the series into the context
				cc[p] = pairs[p].series;
			}
		}

		this.inspect = inspect;
		this.getIterators = getIterators;
	}

	function Label(chart) {
		this.desc = chart;

		function calculate(ctx) {
			var cc = ctx.push();
			this.desc.cc = cc;
			ctx.pop();

			// TODO: what's the minimal size of this bar?
		}

		function render(ctx) {
			var cc = ctx.curr;
			var ch = this.desc;
			var s = ( ch.style != undefined ) ? ch.style : cc.style; 
			var f = ( ch.style != undefined && ch.style.fill != undefined )
				? ch.style.fill
				: (cc.style != undefined ? cc.style.fill : undefined);

			// get by-ref value
			var val = ch.value != undefined ? ch.value.call(cc) : '[none]';
			if (val == null)
				val = '[NULL]';

			var e = new gfx.Text(ctx, val).render().setStroke(s);
			//e.children[0].setFont({size:'16px'});
			if (f != undefined) e.setFill(f);
			// place it to the center of the designated space
			return tr.move(e, (cc.w-e.getTextWidth()-2) / 2, cc.h/2 - 5);
		}

		this.calculate = calculate;
		this.render = render;
	}

	function Bar(chart) {
		this.desc = chart;

		function calculate(ctx) {
			var cc = ctx.push();
			this.desc.cc = cc;
			ctx.pop();

			// TODO: what's the minimal size of this bar?
		}

		function render(ctx) {
			var cc = ctx.curr;
			var ch = this.desc;
			var s = ( ch.style != undefined ) ? ch.style : cc.style; 
			var f = ( ch.style != undefined && ch.style.fill != undefined )
				? ch.style.fill
				: (cc.style != undefined ? cc.style.fill : undefined);
			var e = new gfx.Bar(ctx, cc.w, cc.h).render().setStroke(s);
			if (f != undefined) e.setFill(f);
			return e;
		}

		this.calculate = calculate;
		this.render = render;
	}

	function Overlay(chart) {
		this.desc = chart;

		function prepare(chart) {
			var it = chart.series.getIterator();
			// recurse over all children
			while(it.hasNext()) {
				chartingNG.prepare(it.next());
			}
		}

		function calculate(ctx) {
			// overlay does NOT push any context
			this.desc.cc = ctx;

			// recurse
			var d = this.desc.series.getIterator();
			while (d.hasNext())
				d.next().handler.calculate(ctx);

			//FIXME: recursively collect children info
		}

		function render(ctx) {
			var cc = this.desc.cc;
			var d = this.desc.series.getIterator();
			var oldg = cc.group;
			var g = cc.group = new gfx.Group(ctx).render();
			while (d.hasNext())
				d.next().handler.render(cc);
			cc.group = oldg;
			return g;
		}

		this.prepare = prepare;
		this.calculate = calculate;
		this.render = render;
	}

	function Grid(chart) {
		this.desc = chart;

		function calculate(ctx) {
			// set the current context to the descriptor
			var cc = ctx.push();
			var ch = this.desc;
			ch.cc = cc;

			// ensure .carry series inspection
			new CarryCtx().inspect(cc, ch.carry);

			// inspect/iterate over the control series and recurse
			var colsProps = new SeriesProperties();
			var rowsProps = new SeriesProperties();
			var rows = ch.series['rows'];
			var cols = ch.series['cols'];
			var rows_it = rows.getIterator();
			while(rows_it.hasNext()) {
				rowsProps.mergeElement( rows_it.next() );

				var cProps = new SeriesProperties();
				var cols_it = cols.getIterator();
				while(cols_it.hasNext()) {
					cProps.mergeElement( cols_it.next() );

					// recurse
					ch.what.handler.calculate(cc);
				}
				colsProps.merge(cProps);
			}
			rows.props = rowsProps;
			cols.props = colsProps;

			// TODO: CALCULATE the children minimal sizes and set
			// measurements to be used by render()

			ctx.pop();
		}

		function render(ctx) {
			var ch = this.desc;
			var cc = ch.cc;
			var g = cc.group = new gfx.Group(ctx).render();

			var rows = this.desc.series['rows'];
			var cols = this.desc.series['cols'];

			var cellstyles = ch.series['cell_styles'];
			var widthSeries = new ConstSeries(cc.parent.w/cols.props.count - ch.pad*2);
			var heightSeries = new ConstSeries(cc.parent.h/rows.props.count - ch.pad*2);

			var gridlines = ch.gridlines != undefined
				? ch.gridlines : false;

			new CarryCtx().getIterators(cc, ch.carry);

			/**
			// Grid's bounding rectangle...

			new gfx.Path(cc, "black").render().setStroke(gridlines)
			.moveTo(0,0).hLineTo(cc.parent.w).vLineTo(cc.parent.h)
			.hLineTo(0).closePath();
			 **/

			var cellstylegroup = new gfx.Group(cc).render();

			cc.cellstyles = cellstyles != undefined
				? cellstyles.getIterator() : null;
			cc.heightOffset = new MapIterator(heightSeries.getIterator(),
					function(v, pad) { return v+pad; },
					ch.pad*2 );
			cc.row = rows.getIterator();
			cc.height = heightSeries.getIterator();
			var row = 1;
			while(cc.row.hasNext()) {
				cc.row.next();
				var yoff = cc.heightOffset.next();
				cc.h = cc.height.next(); // propagate inner element height
				cc.widthOffset = new MapIterator(widthSeries.getIterator(),
						function(v, pad) { return v+pad; },
						ch.pad*2 );
				cc.width = widthSeries.getIterator();

				// horizontal grid lines
				if (gridlines && cc.row.hasNext()) {
					var gl = new gfx.Line(cc, cc.parent.w, 0, 0).render();
					tr.move(gl, 0, cc.parent.h - yoff * row);
					gl.setStroke(gridlines);
				}

				cc.col = cols.getIterator();
				var col = 0;
				while(cc.col.hasNext()) {
					cc.col.next();
					var xoff = cc.widthOffset.next();

					if (cc.cellstyles) {
						var ggg = cc.group;
						cc.group = cellstylegroup;
						cc.cellstyle = {width:0, fill: cc.cellstyles.next()};

						// FIXME: HACK need to fix the style setup and other stuff
						var cg = new gfx.Bar(cc, xoff, yoff).render();
						tr.move(cg,	xoff * col, -yoff * row + cc.parent.h);
						cg.setStroke(cc.cellstyle).setFill(cc.cellstyle.fill),
							cc.group = ggg;
					}

					// vertical grid lines
					if (gridlines && row == 1 && col > 0) {
						var gl = new gfx.Line(cc, 0, cc.parent.h, 0).render();
						tr.move(gl, xoff * col, 0);
						gl.setStroke(gridlines);
					}

					cc.w = cc.width.next(); // propagate inner element width

					tr.move(ch.what.handler.render(cc),
							xoff * col + ch.pad,
							-yoff * row + ch.pad + cc.parent.h);
					col++;
				}
				row++;
			}

			return g;
		}

		this.calculate = calculate;
		this.render = render;
	}

	function Columns(chart) {
		this.desc = chart;

		function calculate(ctx) {
			// set the current context to the descriptor
			var cc = ctx.push();
			var ch = this.desc;
			ch.cc = cc;

			// ensure .carry series inspection
			new CarryCtx().inspect(cc, ch.carry);

			// inspect/iterate over the control series and recurse
			var props = new SeriesProperties();
			var heights = ch.series['height'];
			var heights_it = heights.getIterator();
			while(heights_it.hasNext()) {
				props.mergeElement( heights_it.next() );

				// recurse
				ch.what.handler.calculate(cc);
			}
			// check for the case the users used a special series
			if (!heights.props) {
				heights.props = props;
			}

			// TODO: CALCULATE the children minimal sizes and set
			// measurements to be used by render()

			ctx.pop();
		}

		function render(ctx) {
			var ch = this.desc;
			var cc = ch.cc;
			var g = cc.group = new gfx.Group(ctx).render();

			// calculate the max of all segments
			var heights = ch.series['height'];
			values = heights!=null ? heights.getIterator() : null;

			// No series for the column 'height' -> empty group
			if (values == null) {
				return g;
			}
			var dy = cc.parent.h/heights.props.max;

			cc.colors = ch.series['colors'].getIterator();

			new CarryCtx().getIterators(cc, ch.carry);

			// HMMM....
			cc.size = ch.series['size'];
			cc.size = cc.size != undefined
				? cc.size.getIterator() : new IdentityIterator(values);

			var widthSeries = new ConstSeries(cc.parent.w/heights.props.count - ch.pad);
			cc.width = widthSeries.getIterator();
			cc.offset = new MapIterator(widthSeries.getIterator(),
					function(v, pad) { return v+pad; },
					ch.pad );

			// iterate over the chart values
			var n = 0;
			while(values.hasNext()) {
				var h = values.next() * dy;
				cc.width.next();
				cc.size.next();
				cc.style = {fill: cc.colors.next()};

				cc.w = cc.width.peek();
				cc.h = h;

				tr.move(ch.what.handler.render(cc),
						cc.offset.next() * n, 0);
				//FIXME: excluding some values throws off the 'carry' iterators
				//if (values.peek() != null) {
				//}
				n++;
			}

			return g;
		}

		this.calculate = calculate;
		this.render = render;
	}

	function Rows(chart) {
		this.desc = chart;

		function calculate(ctx) {
			// set the current context to the descriptor
			var cc = ctx.push();
			var ch = this.desc;
			ch.cc = cc;

			// ensure .carry series inspection
			new CarryCtx().inspect(cc, ch.carry);

			// inspect/iterate over the control series and recurse
			var props = new SeriesProperties();
			var sizes = ch.series['size'];
			var sizes_it = sizes.getIterator();
			while(sizes_it.hasNext()) {
				props.mergeElement( sizes_it.next() );

				// recurse
				ch.what.handler.calculate(cc);
			}
			sizes.props = props;

			// TODO: CALCULATE the children minimal sizes and set
			// measurements to be used by render()

			ctx.pop();
		}

		function render(ctx) {
			var ch = this.desc;
			var cc = ch.cc;
			var g = cc.group = new gfx.Group(ctx).render();

			// calculate the max of all segments
			var sizeSeries = ch.series['size'];
			sizes = sizeSeries!=null ? sizeSeries.getIterator() : null;

			// No series for the column 'height' -> empty group
			if (sizes == null) {
				return g;
			}
			var dy = cc.parent.h/sizeSeries.props.sum;

			cc.colors = ch.series['colors'].getIterator();

			new CarryCtx().getIterators(cc, ch.carry);

			var offset = 0;
			while(sizes.hasNext()) {
				var h = sizes.next() * dy;

				cc.style = {fill: cc.colors.next()};
				cc.w = cc.parent.w;
				cc.h = h;

				tr.move(ch.what.handler.render(cc), 0, offset);
				//FIXME: excluding some values throws off the 'carry' iterators
				//if (values.peek() != null) {
				//}
				offset += h;
			}

			return g;
		}

		this.calculate = calculate;
		this.render = render;
	}

    function prepare(chart) {
		if (!chart) return null;

		var handler = null;
    	switch(chart.type) {
    	case 'LABEL': 
			handler = new Label(chart);
			break;
    	case 'BAR': 
			handler = new Bar(chart);
			break;
    	case 'grid': 
			handler = new Grid(chart);
			break;
    	case 'columns': 
			handler = new Columns(chart);
			break;
    	case 'rows': 
			handler = new Rows(chart);
			break;
    	case 'overlay': 
			handler = new Overlay(chart);
			break;
    	}

		if (handler && handler.prepare) {
			handler.prepare(chart);
		}
		chart.handler = handler;

		// recurse (chart.what is mostly the only child)
		prepare(chart.what);

		return handler;
    }

    function calculate(ctx, chart, w, h) {
		// set chart.handler recursively
    	prepare(chart);

    	var cc = ctx.push();
    	cc.w = w;
    	cc.h = h;

		// inspect handers and the data series
    	chart.handler.calculate(cc);

    	ctx.pop();
    }

    function render(ctx, chart) {
    	var cc = chart.cc;

    	cc.group = cc.parent.parent.group;
    	cc.w = w;
    	cc.h = h;

		// inspect handers and the data series
    	chart.handler.render(cc);

//    	ctx.pop();
	}

    function chart(ctx, chart, w, h) {
    	calculate(ctx, chart, w, h);
    	render(ctx, chart);
	}

	this.carry = carry;

	/*
	this.render = render;
	*/
	this.prepare = prepare;
	this.chart = chart;
};

var charting_old = {
    setCarry : function(cc, pairs) {
		for(p in pairs) {
			cc[p] = pairs[p].getIterator();
		}
    },
    carry : function(name) {
		return function() {
			// @note this.class = Ctx
			var it = this.get(name);
			if (it.hasNext())
				return it.next();
			return null;
		}
    },

    ecLabel : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);

		// get by-ref value
		var val = chart.value != undefined ? chart.value.call(ctx) : '[none]';
		if (val == null)
			val = '[NULL]';

	    var e = new gfx.Text(ctx, val).render().setStroke(s);
		//e.children[0].setFont({size:'16px'});
		if (f != undefined) e.setFill(f);
		// place it to the center of the designated space
	    return tr.move(e, (c.w-e.getTextWidth()-2) / 2, c.h/2 - 5);
    },

    ecBar : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);
		var e = new gfx.Bar(ctx, c.w, c.h).render().setStroke(s);
		if (f != undefined) e.setFill(f);
		return e;
    },

    ecLine : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
	    var ph = ctx.prev.height;
	    ctx.prev.height = c.height.peek();
	    return new gfx.Line(ctx, c.width.peek(), c.height.peek() - ph).render();
//	    return new gfx.Line(ctx, -c.width.peek() / 2, ph - c.height.peek(),
//                c.width.peek(), c.height.peek() - ph, s
//                        .peek()).render();
    },

    ecEllipse : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);
		var e = new gfx.Ellipse(ctx, c.w/2, c.h/2).render().setStroke(s);
		// place it to the center of the designated space
		var e = tr.move(e, c.w/2, c.h/2);
		if (f != undefined) e.setFill(f);
		return e;
    },

    ecPieSlice : function(ctx, chart) {
	    var c = ctx.curr;
		var s = ( chart.style != undefined ) ? chart.style : c.style; 
		var f = ( chart.style != undefined && chart.style.fill != undefined )
					? chart.style.fill
					: (c.style != undefined ? c.style.fill : undefined);
		var e = new gfx.PieSlice(ctx, c.angle.peek(), c.radius.peek()).render()
					.setStroke(s);
		if (f != undefined) e.setFill(f);
		return e;
    },

    ecPie: function(ctx,chart) {
    	var d = chart.series['width'];
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

    	// calculate the sum of all segments
    	var props = inspect( d.getIterator());
    	if (props.sum <= 0) {return '';}

    	c.colors = chart.series['colors'].getIterator();

    	// render separate slices
    	c.width = d.getIterator();
    	c.angle = new MapIterator( d.getIterator(), function(v, sum) { return Math.round((v*360)/sum); }, props.sum );
    	// c.radius = new ConstIterator(c.parent.w/2);
    	//c.radius = new MapIterator(d.getIterator(), function(v, p) { return v*p.w/p.sum; }, {w: c.parent.w, sum: props.sum} );

    	var dummy = function(i) {
    		//alert(i);
    	};

		// FIXME: HACK get the radius from a provided dimension rather then making it up
    	c.radius = new MapIterator(d.getIterator(), function(v, p) {
    								dummy(p.size + "/" + p.sum + " = " + p.w*(p.size/p.sum)); return p.w/2*v; },
    								{size: 2, w: c.parent.w, sum: props.sum} );
    								//{size: c.parent.size.peek(), w: c.parent.w, sum: props.sum} );
    	//c.radius = new MapIterator(d.getIterator(), function(v, p) { return p.w*(p.size/7); }, {size: c.parent.size.peek(), w: c.parent.w, sum: props.sum} );

    	var ret = c.group;
    	while(c.angle.hasNext()) {
    		c.width.next();
    		c.radius.next();
    		c.style = {fill: c.colors.next()};
    		var a = c.angle.next();

    		// rotate the parent group of slices
    		tr.rotate(c.group,a);

    		//Switch the g and c.group in the DOM
    		// parent -> g -> c.group
    		// parent -> c.group -> g
    		g = c.group;
    		var parent = g.getParent();
    		c.group = new gfx.Group(ctx).render();
    		c.group.removeShape();
    		parent.add(c.group);
    		c.group.add(g);
    		ret = c.group;
    		
    		// draw the current slice into its own subgroup
    		this.ec(ctx, chart.what);
    	}
    	ctx.pop();
    	return ret;
    },

    ecShape : function(ctx, chart) {
    	switch(chart.which) {
    	case 'line': 
    		return this.ecLine(ctx, chart);
    	case 'bar': 
    		return this.ecBar(ctx, chart);
    	case 'ellipse': 
    		return this.ecEllipse(ctx, chart);
		}
    },

    ecColumns: function(ctx, chart) {
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

    	// calculate the max of all segments
    	var d = chart.series['height'];
    	values = d!=null ? d.getIterator() : null;

		// No series for the column 'height' -> empty group
		if (values == null) {
    		ctx.pop();
			return g;
		}
    	var props = inspect(values);
    	var dy = c.parent.h/props.max;

    	ctx.prev = {};
    	c.colors = chart.series['colors'].getIterator();

		this.setCarry(c, chart.carry);

    	values = d.getIterator();

    	// HMMM....
    	c.size = chart.series['size'];
    	c.size = c.size != undefined
					? c.size.getIterator() : new IdentityIterator(values);

    	// AVOID: using multiple iterators over the same series
    	//        it could confuse the ControlledSeries
    	//
    	//c.height = new MapIterator(
		//					d.getIterator(),
    	//					function(v, p) { return v*p.dy+p.offset; },
    	//					{ dy:dy, offset:500 });
    	var widthSeries = new ConstSeries(c.parent.w/props.count - chart.pad);
    	c.width = widthSeries.getIterator();
    	c.offset = new MapIterator(widthSeries.getIterator(),
    	                           function(v, pad) { return v+pad; },
    	                           chart.pad );
    	// c.width = new MapIterator(
		//				new ConstSeries(60).getIterator(),
		//				function(v, seq) { seq.value++; return v*seq.value; },
		//				{ value:0 } );

    	// iterate over the chart values
    	var ret = c.group;
    	var n = 0;
    	while(values.hasNext()) {
    		var h = values.next() * dy;
    		c.width.next();
    		c.size.next();
    		c.style = {fill: c.colors.next()};

    		c.w = c.width.peek();
    		c.h = h;

				tr.move(this.ec(ctx, chart.what),
						c.offset.next() * n, 0);
			//FIXME: excluding some values throws off the 'carry' iterators
			//if (values.peek() != null) {
			//}
			n++;
		}

		ctx.pop();
    	return ret;
    },

    ecRows: function(ctx, chart) {
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

    	// calculate the max of all segments
    	var d = chart.series['size'];
    	sizes = d!=null ? d.getIterator() : null;

		// No series for the column 'height' -> empty group
		if (sizes == null) {
    		ctx.pop();
			return g;
		}
    	var props = inspect(sizes);
    	var dy = c.parent.h/props.sum;

    	c.colors = chart.series['colors'].getIterator();

		this.setCarry(c, chart.carry);

    	sizes = d.getIterator();

    	var offset = 0;
    	while(sizes.hasNext()) {
    		var h = sizes.next() * dy;

    		c.style = {fill: c.colors.next()};
    		c.w = c.parent.w;
    		c.h = h;

				tr.move(this.ec(ctx, chart.what), 0, offset);
			//FIXME: excluding some values throws off the 'carry' iterators
			//if (values.peek() != null) {
			//}
			offset += h;
		}

		ctx.pop();
    	return g;
    },

    ecScattered: function(ctx, chart, lines) {
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;

		// FIXME: These should be calculated from the size of the
		//        markers + some defaults to get 'padding'
		var xmin = c.parent.w/15;
		var ymin = c.parent.w/15;

    	// calculate the max of all segments
    	var x = chart.series['x'];
    	var y = chart.series['y'];

    	var propsX = inspect(x.getIterator());
    	var dx = (c.parent.w-xmin*2)/propsX.max;

    	var propsY = inspect(y.getIterator());
    	var dy = (c.parent.h-ymin*2)/propsY.max;

    	c.x = x.getIterator();
    	c.y = y.getIterator();
		c.style = chart.style;

    	// iterate over the chart values
		var ret = c.group;
		var path = (lines)
					? new gfx.Path(ctx).render().setStroke(c.style) : null;
		var n = 0;
		while(c.x.hasNext() && c.y.hasNext()) {
			var px = xmin + c.x.next() * dx;
			var py = ymin + c.y.next() * dy;

			if (c.x.peek() != null && c.y.peek() != null) {
				if (lines) {
					if (n > 0)
						path.lineTo(px,py);
					else
						path.moveTo(px,py);
				}

				c.w = Math.min(c.parent.w - px, px);
				c.h = Math.min(c.parent.h - py, py);
				// FIXME: Hack: how big should the markers be?
				c.w = c.parent.w/12;
				c.h = c.parent.w/12;

				tr.move(this.ec(ctx, chart.what), px-c.w/2, py-c.h/2);
			}
			n++;
		}
		ctx.pop();
    	return ret;
    },


    ecGrid: function(ctx, chart) {
    	var cols = chart.series['cols'];
    	var rows = chart.series['rows'];
    	var cellstyles = chart.series['cell_styles'];
    	var g = new gfx.Group(ctx).render();
    	var c = ctx.push();
    	c.group = g;
    	var propsC = inspect( cols.getIterator());
    	var propsR = inspect( rows.getIterator());
    	var widthSeries = new ConstSeries(c.parent.w/propsC.count - chart.pad*2);
    	var heightSeries = new ConstSeries(c.parent.h/propsR.count - chart.pad*2);

    	var gridlines = chart.gridlines != undefined
							? chart.gridlines : false;

		this.setCarry(c, chart.carry);

		var ret = c.group;

		/**
		// Grid's bounding rectangle...

		new gfx.Path(ctx, "black").render().setStroke(gridlines)
				.moveTo(0,0).hLineTo(c.parent.w).vLineTo(c.parent.h)
				.hLineTo(0).closePath();
		**/

    	var cellstylegroup = new gfx.Group(ctx).render();

		c.cellstyles = cellstyles != undefined
							? cellstyles.getIterator() : null;
    	c.heightOffset = new MapIterator(heightSeries.getIterator(),
    	                           function(v, pad) { return v+pad; },
    	                           chart.pad*2 );
		c.row = rows.getIterator();
		c.height = heightSeries.getIterator();
		var row = 1;
		while(c.row.hasNext()) {
			c.row.next();
			var yoff = c.heightOffset.next();
			c.h = c.height.next(); // propagate inner element height
			c.widthOffset = new MapIterator(widthSeries.getIterator(),
											function(v, pad) { return v+pad; },
											chart.pad*2 );
			c.width = widthSeries.getIterator();

			// horizontal grid lines
			if (gridlines && c.row.hasNext()) {
				var gl = new gfx.Line(ctx, c.parent.w, 0, 0).render();
				tr.move(gl, 0, c.parent.h - yoff * row);
				gl.setStroke(gridlines);
			}

			c.col = cols.getIterator();
			var col = 0;
			while(c.col.hasNext()) {
				c.col.next();
				var xoff = c.widthOffset.next();

				if (c.cellstyles) {
					var ggg = c.group;
					c.group = cellstylegroup;
					c.cellstyle = {width:0, fill: c.cellstyles.next()};

					// FIXME: HACK need to fix the style setup and other stuff
					var cg = new gfx.Bar(ctx, xoff, yoff).render();
					tr.move(cg,	xoff * col, -yoff * row + c.parent.h);
					cg.setStroke(c.cellstyle).setFill(c.cellstyle.fill),
					c.group = ggg;
				}

				// vertical grid lines
				if (gridlines && row == 1 && col > 0) {
					var gl = new gfx.Line(ctx, 0, c.parent.h, 0).render();
					tr.move(gl, xoff * col, 0);
					gl.setStroke(gridlines);
				}

				c.w = c.width.next(); // propagate inner element width

				// carry for dependent data series
				this.setCarry(c, chart.depcarry);

				tr.move(this.ec(ctx, chart.what),
						 xoff * col + chart.pad,
						-yoff * row + chart.pad + c.parent.h);
				col++;
			}
			row++;
		}

    	ctx.pop();
    	return ret;
    },


    ecOverlay : function(ctx, chart) {
    	var d = chart.series.getIterator();
    	var oldg = ctx.curr.group;
    	ctx.curr.group = new gfx.Group(ctx).render();
    	ret = ctx.curr.group;
    	while (d.hasNext()) {
    		this.ec(ctx, d.next());
    	}
    	ctx.curr.group = oldg;
    	return ret;
    },

    ec: function(ctx, chart) {
		if (!chart) return null;

    	switch(chart.type) {
    	case 'shape': 
    		return this.ecShape(ctx, chart);
    	case 'LINE': 
    		return this.ecLine(ctx, chart);
    	case 'BAR': 
    		return this.ecBar(ctx, chart);
    	case 'ELLIPSE': 
    		return this.ecEllipse(ctx, chart);
    	case 'PIESLICE': 
    		return this.ecPieSlice(ctx, chart);
    	case 'LABEL': 
    		return this.ecLabel(ctx, chart);
    	case 'pie': 
    		return this.ecPie(ctx, chart);
    	case 'grid': 
    		return this.ecGrid(ctx, chart);
    	case 'scattered': 
    		return this.ecScattered(ctx, chart, false);
    	case 'line': 
    		return this.ecScattered(ctx, chart, true);
    	case 'columns': 
    		return this.ecColumns(ctx, chart);
    	case 'rows': 
    		return this.ecRows(ctx, chart);
    	case 'overlay': 
    		return this.ecOverlay(ctx, chart);
    	}
    	return null;
    },

    ecc: function(ctx, chart, w, h) {
    	var c = ctx.push();

    	c.group = c.parent.group;
    	c.w = w;
    	c.h = h;

    	var ret = this.ec(ctx, chart);

    	ctx.pop();
    	return ret;
    }
}


var w = 800;
var h = 600;

var regions = new Series(['Central','East','South','West']);
var prod_types = new Series(['Coffe','Espresso','Herbal Tea','Tea']);

var coffe_prods = new Series(['Amaretto','Columbia','Decaf']);
var espresso_prods = new Series(['Cafe Latte','Mocca','Decaf Espresso', 'Espresso']);
var herbaltea_prods = new Series(['Cammomille','Lemon','Mint']);
var tea_prods = new Series(['Darjeeling','Earl Grey','Green Tea']);

var products = new Series([coffe_prods, espresso_prods, herbaltea_prods, tea_prods]);

var prod_types_it = new IteratingSeries(prod_types);
var regions_it = new IteratingSeries(regions);

var sales_c_c = new Series([14000,28900,26000]);
var sales_c_e = new Series([2994,47385,6262]);
var sales_c_s = new Series([null,21663,11596]);
var sales_c_w = new Series([9230,30300,18233]);

var sales_e_c = new Series([null,35000,24000,null]);
var sales_e_e = new Series([null,16646,7720,24000]);
var sales_e_s = new Series([15443,14166,15381,null]);
var sales_e_w = new Series([20466,18874,30578,null]);

var sales_coffe    = new Series([sales_c_c, sales_c_e, sales_c_s, sales_c_w]);
var sales_espresso = new Series([sales_e_c, sales_e_e, sales_e_s, sales_e_w]);

var sales = new Series([new ControlledSeries(sales_coffe, regions_it),
						new ControlledSeries(sales_espresso, regions_it)]);

var sales_ce = new Series([sales_coffe, sales_espresso]);


prod_types = new Series(['Coffe','Espresso']);//,'Herbal Tea','Tea']);
prod_types_it = new IteratingSeries(prod_types);

var cols_prodtype_regionNG = {
	type :"columns",
	pad :10,
	series : {
		height: new InspectedSeries(new ControlledSeries(sales, prod_types_it), [prod_types_it, regions_it]),
		// FIXME: TODO: height: charting.carry('val'),
		colors: new Series(['lightskyblue','lightcoral','lightgreen',
							'lightyellow','lightcyan','lightblack'])
		// legend: new ControlledSeries(products, prod_types_it)
	},
	carry: {
		val: {
			series: new ControlledSeries(sales, prod_types_it),
			over: [prod_types_it, regions_it]
		}
	},
	what1 :{type :"LABEL",
			style:{width:3, color:'gray'},
			value: chartingNG.carry('val')
	},
	what : {
		type :"overlay",
		series: new Series([{	type :"BAR",
								style:{width:3, color:'gray'}
							},
							{	type :"LABEL",
								style:{width:0, fill:'darkblue'},
								value: chartingNG.carry('val')
							}])
	}
}

var colgridNG = {
    type :"grid",
    pad :5,
    gridlines: {width:1, color:"blue"},
    series : {
        cols: prod_types_it,
        rows: regions_it,
    },
    what : cols_prodtype_regionNG
}

var colgridgridNG = {
    type: "grid",
    pad: 10,
    gridlines: {width:2, color:"green"},
    series : {
		cols: new Series([1,2]),
		rows: new Series([1]),
		cell_styles: cellstyles
    },
	what: colgridNG
}


var rowsNG = {
	type :"rows",
	series : {
		size: new Series([1,2,3,4]),
		colors: new Series(['lightskyblue','lightcoral','lightgreen'])
	},
	carry: {
		val: {
			series: new ControlledSeries(sales, prod_types_it),
			over: [prod_types_it, regions_it]
		}
	},
	what1 :{type :"BAR",
			style:{width:3, color:'gray'},
			value: chartingNG.carry('val')
	},
	what : {
		type :"overlay",
		series: new Series([{	type :"BAR",
								style:{width:3, color:'gray'}
							},
							{	type :"LABEL",
								style:{width:0, fill:'darkblue'},
								value: chartingNG.carry('val')
							}])
	}
}

/*
w = w/3;
h = h/3;
*/
var ctx = new Ctx();
	ctx.push().group = new gfx.Surface('svgdivNGc1', w, h).render();

prod_types_it.getIterator().next();
regions_it.getIterator().next();
var c5 = chartingNG.chart(ctx,colgridgridNG,w,h);

//var c5 = chartingNG.chart(ctx,rowsNG,w,h);


// vim: ai:ts=4:sw=4:tw=0
