dojo.require("dojo.colors"); // to have all the CSS3 color definitions 
dojo.require("dojox.gfx"); 

var m = dojox.gfx.matrix;
var stroke = {color:'gray', width:3};
var color = 'gray';

var tr = 
{
	move : function(what, x, y) {
		if (what == null) return null;
		what.applyTransform(m.translate(x,y));
	    return what;
    },
	rotate : function(what, angle) {
		if (what == null) return null;
	    what.applyTransform(m.rotateg(angle));
	    return what;
    },
	scale : function(what, dx, dy) {
		if (what == null) return null;
	    what.applyTransform(m.scale(dx, dy));
	    return what;
    }
}

var gfx = {
	Surface : function(name, w, h) {
		this.render = function() {
			var surface = new dojox.gfx.createSurface(document.getElementById(name), w, h);
			var grp = surface.createGroup().setStroke(stroke);
			return tr.move(tr.scale(grp, 1,-1), 0, -h); // fix the coordinate system bottom left
		}
	},
	Group : function(ctx) {
		this.render = function() {
			return ctx.curr.group.createGroup();
		}
	},

    Ellipse : function(ctx, rx, ry) {
	    this.render = function() {
	    	rx= rx<=0?1:rx; // necessary for VML (invalid string init Vector2D error)
	    	ry= ry<=0?1:ry; // as it does not support non-positive width and height nodes
		    return ctx.curr.group.createEllipse({cx:0, cy:0, rx:rx, ry:ry});
	    }
    },
    Line : function(ctx, w, h) {
	    this.render = function() {
		    return ctx.curr.group.createLine({x1:0, y1:0, x2:w, y2:h});
	    }
    },
    Bar : function(ctx, w, h) {
	    this.render = function() {
	    	h= h<=0?1:h;
	    	w= w<=0?1:w;
		    return ctx.curr.group.createRect({x:0, y:0, width:Math.round(w), height:Math.round(h)});
	    }
    },
    Text : function(ctx, text, c) {
	    this.render = function() {
		    // construct a text object and invert its Y axis as we use different
		    // coordinate system
			var g = ctx.curr.group.createGroup();
			var t = tr.scale(g.createText({text:text}), 1, -1);
			// enclosed in a group and redirect the property operation to the text node
			// others are done on top of the enclosing group (transformations
			// behave differently for the invert Y axis coordinate system
			g.getTextWidth = function() { return this.children[0].getTextWidth(); };
			g.setStroke = function(s) { this.children[0].setStroke(s); return this; };
			g.setFill = function(f) { this.children[0].setFill(f); return this; };
			return g;
	    }
    },
    Path : function(ctx) {
	    this.render = function() {
		    return ctx.curr.group.createPath();
	    }
    },
    PieSlice : function(ctx, a, r, c) {
	    this.render = function() {
		    var x2 = Math.cos((a / 180) * Math.PI) * r;
		    var y2 = Math.sin((a / 180) * Math.PI) * r;
		    r = Math.round(r);
		    x2 = Math.round(x2);
		    y2 = Math.round(y2);
		    return ctx.curr.group.createPath().setAbsoluteMode(false)
		    	   .moveTo(0,0)
		    	   .lineTo(r,0)
		    	   .arcTo(r,r,0,a>180,true,{x:x2-r, y:y2})
		    	   .closePath();
	    }
    }
}

// vi: ai:ts=4:sw=4:tw=0
