var GFX_MIN = -1000; 
var GFX_MAX = 1000;
var GFX_RANGE = GFX_MAX - GFX_MIN;

function dump(o, n, indent, visited) {
	indent = indent ? indent : '';
	n = n ? n : '';

	var r = '';
	if (o === null) {
		r = indent + n + ' null\n';
	} else if (typeof o == 'object') {
		visited = visited ? visited : new Array();
		var idx = visited.indexOf(o);
		if (idx>=0)
			return indent + n + ' ['+visited.indexOf(o)+']\n';
		visited.push(o);
		idx = visited.indexOf(o);

		r = indent + n + '{['+idx+']\n';
		for(i in o) {
			r += dump(o[i], i, indent + '  ', visited);
		}
		r += indent + '}\n';
	} else if (typeof o == 'function') {
//		r = indent + n + ' function\n';
	} else {
		r = indent + n + ' ' + o + '\n';
	}

	return r;
}

/**
 * Hierarchical context class.
 * @param parent Parent context object
 * @return class
 */
function Ctx(parent) {
	this.parent = parent;
	this.curr = this;

	this.push = function() {
		this.curr = new Ctx(this.curr);
		return this.curr;
	}

	this.pop = function() {
		return this.curr = this.curr.parent;
	}

	/**
	 * Put value into the current context. Similar to what one
	 * can do with ctx.push().name = value.
	 **/
	this.put = function(name, value) {
		this.curr[name] = value;
	}
	this.get = function(name) {
		var cc = this.curr;
		while (cc) {
			var cv = cc[name];
			if (cv != undefined) return cc;
			cc = cc.parent;
		}
	}
}

function drawArea(obj, w, h) {
	var ctx = new Ctx();
	var cc = ctx.push();
	cc.x = 0;
	cc.y = 0;
	cc.w = w;
	cc.h = h;
	draw(obj,ctx);
	ctx.pop();
}

/**
 * Constructs the actual coordinates representation of the graphical element. 
 * @param obj Needs to have the 'rendering' function set to the 'what' property
 * @param ctx Context object to propagate data
 * @return Graphical element representation using real coordinates
 */
function draw(obj, ctx) {
	ctx.curr.obj = obj;
	var r = obj.what.call(obj,ctx);
}

function bar(ctx)
{
	var cc = ctx.curr;
	var bar = {
		id: this.id,
		x: cc.x,
		y: cc.y,
		w: this.w * cc.w / GFX_MAX,
		h: this.h * cc.h / GFX_MAX
	}
	print(dump(bar));
	return bar;
}

function pieslice(ctx)
{
	var cc = ctx.curr;
	var slice = {
		id: this.id,
		x: cc.x,
		y: cc.y,
		a: Math.round( (this.a*360) / GFX_MAX ),
		o: Math.round( (cc.offset*360) / GFX_MAX ),
		r: this.r * Math.min(cc.w, cc.h) / 2 / GFX_MAX
	}
	print(dump(slice));
	return slice;
}

function adjustXYWH(cc, pc, obj) {
	cc.x = pc.x + (obj.x * pc.w / GFX_MAX);
	cc.y = pc.y + (obj.y * pc.h / GFX_MAX);
	cc.w = obj.w * pc.w / GFX_MAX;
	cc.h = obj.h * pc.h / GFX_MAX;
}

function pie(ctx) {
	var pc = ctx.curr;
	var cc = ctx.push();
	adjustXYWH(cc, pc, this);
	cc.offset = 0;

	var ch = this.children;
	for(var p in ch) {
		var slice = ch[p];
		draw(slice, ctx);
		cc.offset += slice.a;
	}

	ctx.pop();
}

//just draw all children
function composite(ctx) {
	var ch = this.children;

	var pc = ctx.curr;
	var cc = ctx.push();
	cc.w = pc.w;
	cc.h = pc.h;
	for(var p in ch) {
		draw(ch[p], ctx);
	}
	ctx.pop();
}

//just draw all children
function columns(ctx) {
	var pc = ctx.curr;
	var cc = ctx.push();
	adjustXYWH(cc, pc, this);

	var cols = {
		id: this.id,
		x: cc.x,
		y: cc.y,
		w: cc.w,
		h: cc.h
	}
	print(dump(cols));

	var ch = this.children;
	for(var p in ch) {
		draw(ch[p], ctx);
	}

	ctx.pop();
}


var ps1 = {
	what :pieslice,
	id:'ps1',
	a :150,
	r :200
}

var ps2 = {
	what :pieslice,
	id:'ps2',
	a :250,
	r :800
}

var ps3 = {
	what :pieslice,
	id:'ps3',
	a :600,
	r :800
}

var pie = { 
	what: pie,
	id:'pie1',
	x: GFX_MAX/4,
	y: GFX_MAX/2,
	w: GFX_MAX,
	h: GFX_MAX,
	children: [ps2, ps1, ps3]
}



var bar11 = {
	what :bar,
	id :'bar11',
	w :GFX_MAX / 2,
	h :GFX_MAX / 3
}

var bar12 = {
	what :bar,
	id :'bar12',
	w :GFX_MAX / 2,
	h :GFX_MAX / 1.5
}

var bar21 = {
	what :bar,
	id :'bar21',
	w :GFX_MAX / 2,
	h :GFX_MAX
}

var bar22 = {
	what :bar,
	id :'bar21',
	w :GFX_MAX / 2,
	h :GFX_MAX / 4
}

var cols1 = {
	what :columns,
	id :'cols1',
	x :0,
	y :0,
	w :GFX_MAX / 2,
	h :GFX_MAX,
	children: [bar11, bar12]
}

var cols2 = {
	what :columns,
	id :'cols2',
	x :0,
	y :0,
	w :GFX_MAX / 2,
	h :GFX_MAX / 10,
	children: [bar21, bar22]
}

var colsN = {
	what :columns,
	id :'colsN',
	x :0,
	y :0,
	w :GFX_MAX,
	h :GFX_MAX,
	children : [ cols1, cols2 ]
}

// vim: ai:ts=4:sw=4:tw=0
